// singleton pattern by Addy Osmani
var FoodDiaryWizard = (function () {

  // Instance stores a reference to the Singleton
  var instance;
  var TYPES = ["photo", "photolibrary", "text"];
  var DUMMY_PIC = false;

  function init(options) {
    // private variabless
    var options = options;
    var mode = "";
    var type = "";
    var paramString = "";
    var fromPage = "";

    // Singleton
    function setCameraOptions(srcType) {
      return {
        // Some common settings are 20, 50, and 100
        quality: 20,
        destinationType: Camera.DestinationType.FILE_URI,
        targetWidth: 1024,
        targetHeight: 1024,
        // In this app, dynamically set the picture source, Camera or photo gallery
        sourceType: srcType,
        encodingType: Camera.EncodingType.JPEG,
        mediaType: Camera.MediaType.PICTURE,
        allowEdit: false,
        correctOrientation: true,  //Corrects Android orientation quirks
        saveToPhotoAlbum: true
      }
    }

    function openCamera(gotoPage, paramString, library) {
      try {
        if (library == null) {var srcType = Camera.PictureSourceType.CAMERA;}
        else {var srcType = Camera.PictureSourceType.PHOTOLIBRARY;}

        var cameraOptions = setCameraOptions(srcType);
        navigator.camera.getPicture(function cameraSuccess(imageUri) {
          $j.mobile.changePage("#"+gotoPage+"?"+paramString);
          $j("#"+options.displayImage).attr("src", imageUri);
        }, function cameraError(error) {
        }, cameraOptions);
      } catch (e) {
        if (DUMMY_PIC) {
          console.log("Camera not found! Will use dummy picture");
          console.log("change page to "+"#"+gotoPage+"?"+paramString);
          $j.mobile.changePage("#"+gotoPage+"?"+paramString);
          $j("#"+options.displayImage).attr("src", "img/dummyfood.jpg");
        } else {
          showAlert("Fout: camera niet gevonden", "Dit apparaat heeft geen camera of deze kan niet worden gebruikt");
        }
      }
    }

    function win(r) {
       console.log("Code = " + r.responseCode);
       console.log("Response = " + r.response);
       console.log("Sent = " + r.bytesSent);
       console.log("finish with result= "+r);
       options.onFinish(r);
    }

    function fail(error) {
      alert("An error has occurred: Code = " + error.code);
    }

    function updateWizardStyles(type) {
      console.log("update style "+type+ "- options.endpage="+options.endPage);
      for (var i=0; i<TYPES.length; i++) {
        $j("#"+options.endPage).removeClass(TYPES[i]);
      }
      if (type=='photolibrary') {$j("#"+options.endPage).addClass('photo');}
      else {$j("#"+options.endPage).addClass(type);}
    }

    function makePicture() {
      openCamera(options.endPage, paramString);
    }

    function selectPicture() {
      var library = 'library'
      openCamera(options.endPage, paramString, library);
    }

    return {
      // Public functions here
      onSelectFoodMoment: function() {
        switch(type) {
          case "text":
            console.log("param string:"+paramString)
            $j.mobile.changePage("#"+options.endPage+"?"+paramString);
            break;
          case "photo":
            makePicture();
            break;
          case "photolibrary":
            selectPicture();
            break;
        }
      },

      start: function(the_type, the_mode, the_paramString) {
        if (typeof(the_mode)=="undefined") the_mode="create"
        mode = the_mode;
        type = the_type
        paramString = the_paramString;

        updateWizardStyles(type);

        fromPage = $j.mobile.activePage.attr('id');

        switch(mode) {
          case "create":
            $j.mobile.changePage("#"+options.startPage);
            break;
          case "edit":
            console.log($j.parseParams(paramString));
            $j.mobile.changePage("#"+options.endPage+"?"+paramString);
        }
      },

      cancel: function() {
        console.log("cancel "+paramString);
         $j.mobile.changePage("#"+fromPage+"?"+paramString);
      },

      end: function() {

        uploadParams = {
          id: $j("#"+options['fieldPrefix']+'Id').val(),
          moment: $j("#"+options['fieldPrefix']+'Moment').val(),
          description: $j("#"+options['fieldPrefix']+'Description').val(),
          consumed_at: $j("#"+options['fieldPrefix']+'ConsumedAt').val(),
          consumed_on: $j("#"+options['fieldPrefix']+'ConsumedOn').val()
        }

        switch(mode) {
          case "create":
            switch(type) {
              case "photolibrary":
              case "photo":
                try {
                  console.log("store "+type+","+mode);
                  this.uploadPhoto(uploadParams);
                } catch(e) {
                  alert("Er is een fout opgetreden tijdens het uploaden."+e);
                }
                break;
              case "text":
                doPost(
                  options.uploadURL,
                  uploadParams,
                  true,
                  function(data) {win(data.result);},
                  function(error) {
                    $j("#"+options['fieldPrefix']+'Description').addClass('error');
                    enableButton("#submit");
                  }
                );
                break;
            }
            break;
          case "edit":
            doPut(
              options.uploadURL,
              uploadParams,
              true,
              function(data) {win(data.result);},
              function(error) {
                $j("#"+options['fieldPrefix']+'Description').addClass('error');
                   enableButton("#submit");
              }
            );
            break;
        }
      },

      uploadPhoto: function(uploadParams) {
        var loginName = getLoginName();
        var password = getPassword();
        imageURI = $j("#"+options.displayImage).attr("src");
        var uploadOptions = new FileUploadOptions();
        uploadOptions.fileKey = "file";
        uploadOptions.fileName = imageURI.substr(imageURI.lastIndexOf('/')+1);
        uploadOptions.mimeType = "image/jpeg";
        uploadOptions.chunkedMode = false;
        uploadOptions.headers = {
          "X-ECOACHING-EMAIL": btoa(loginName),
          "X-ECOACHING-PASSWORD": btoa(password)
        },
        uploadOptions.params = {
          filename : uploadOptions.fileName,
          mime_type : uploadOptions.mimeType
        }
        new FileTransfer().upload(imageURI, options.uploadURL+"?"+$j.param(uploadParams),
          function(r) {win($j.parseJSON(r.response).result)},
          fail, uploadOptions);
      },

      getOptions() {
        return options;
      },

      getMode() {
        return mode;
      },

      getParamString() {
        return paramString;
      },

      getFromPage() {
        return fromPage;
      }
    };
  };

  return {

    // Get the Singleton instance if one exists
    // or create one if it doesn't
    getInstance: function (options) {

      if ( !instance ) {
        instance = init(options);
      }

      return instance;
    }

  };

})();
